from base_class import Calculator
from handler_class import HandlerCalculator

base = Calculator(10)
handler = HandlerCalculator()

base.screen()
base.divide
base.screen()


# handler.screen()
# handler.reset(5)
# print(handler.memory)
# handler.screen()
# handler.add(4, 5, only=True)
# print(handler.memory)
# handler.add(5, 5, 5)
# handler.screen()
# handler.nth_root(2, -5)